%jpb CEST/CPMG SQ,MQ global fitting (2-/3-state BAC). 3-state model: A (major)<-> B and A (major)<-> C (linear)
% used for locally fitting 2-state/3-state (L259,V58) probes in apo hTS
% global fitting used only for geminal methyls of the same residue

clear;
clear global field;
clear global taucp;
clear global data;
clear global n180array;
clear global delta;
clear global datainput;
clear global dataout;

global field
global protonfield
global taucp
global data
global n180array
global delta
global ptsmax
global isMQ
global numres
global freq
global sllist
global sllist_allres
global datainput
global numpts
global chemshift
global fityn
global dataout
global SLdura
global chemcenter
global relErr
global idxlist

options = optimset('lsqnonlin');
options = optimset(options,'algorithm','trust-region-reflective');
%options = optimset(options,'algorithm','levenberg-marquardt');
options = optimset(options,'Display','off');
options = optimset(options,'TolFun',1E-12);
options = optimset(options,'TolX',1E-12);
options = optimset(options,'MaxFunEvals', 10000);
%options = optimset(options,'FinDiffRelStep',1e-3);

% load CPMG mats (from profile plotting script)
load('R2eff_MQ850.mat')
load('eR2eff_MQ850.mat')
load('R2eff_MQ600.mat')
load('eR2eff_MQ600.mat')
load('R2eff_SQ850.mat')
load('eR2eff_SQ850.mat')
load('taucpmg_MQ850.mat')
load('taucpmg_MQ600.mat')
load('taucpmg_SQ850.mat')

%load CEST mats (from profile plotting/spin lock calibration scripts)
load('CEST25_full2_mat.mat')
load('CEST40_full3_mat.mat')
load('powcalib40.mat')
load('calib25pow.mat')

%residue = {'L192','L198','L187','L121','L101 1','L101 2','L131 1','L131 2','L212','L252','L73 1','L73 2','L74','V79','L221','L259','L269','V164'};
residue = {'L73 1','L73 2'}; %profile name(s) for plotting (single probe or two geminals of same residue)
dw0 = [2 3]; %inital guess of 13C dw (slow process if 3-state)
dwh0 = [0 0]; %inital guess of 1H dw (slow process if 3-state)
mc = 200; %number of mc sims

numres = length(residue);

isMQ = [1 1 0]; %1 = MQ, 0 = SQ

is3state = 0; %1 = 3 state, 0 = 2 state

ylolim = 0;     % lower limit for y-axis in plot

% set up initial param array, bounds
if is3state == 1
    xlb = [0.6 0.95 1 20]; % lower bounds for 
    xub = [0.99 1 1000 15000]; % upper bounds for fitted params
    x0 = [0.98 0.99 99 1000];
    %pa,pa+pb,kexab,kexac

    xlb_res = [-5 -5 0 0 5 5 5 0 0];
    xub_res = [5 5 0 0 80 80 80 5 50];
    %dwcab,dwcac,dwhab,dwhac,r2abc1,r2abc2,r2abc3,CEST R1,CEST R2
else
    xlb = [0.6 20]; % lower bounds for 
    xub = [1 10000]; % upper bounds for fitted params
    x0 = [0.98 2000];
    %pa,pa+pb,kexab,kexac

    xlb_res = [0 0 5 5 5 0 0];
    xub_res = [5 0 80 80 80 5 50];
    %dwcab,dwcac,dwhab,dwhac,r2abc1,r2abc2,r2abc3,CEST R1,CEST R2
end

for i = 1:numres
    xlb = [xlb xlb_res];
    xub = [xub xub_res];
end

%set up CPMG stuff
flds = 3;

%13C field strength (MHz)
field = [213.802506 150.902406 213.802506];

ptsmax = [length(taucpmg_MQ850) length(taucpmg_MQ600) length(taucpmg_SQ850)];

protonfield = field*(850.28/213.802506); %assumes heteronuc is 13C

taucp{1} = taucpmg_MQ850';
taucp{2} = taucpmg_MQ600';
taucp{3} = taucpmg_SQ850';

%index values for profile(s) to be fit
%for example, 21(MQ), 23(SQ), 39 (CEST) for L198
idxMQ = [16 57];
idxSQ = [17 61];
idxlist = [35 75]; %CEST

for i = 1:numres
    data{i,1} = [R2eff_MQ850(idxMQ(i),:)' eR2eff_MQ850(idxMQ(i),:)'];
    data{i,2} = [R2eff_MQ600(idxMQ(i),:)' eR2eff_MQ600(idxMQ(i),:)'];
    data{i,3} = [R2eff_SQ850(idxSQ(i),:)' eR2eff_SQ850(idxSQ(i),:)'];
  
    %intial guess of R2 (CPMG) for each field
    r2abc(i,:) = [min(data{1}(:,1)) min(data{2}(:,1)) min(data{3}(:,1))];

    for j = 1:flds
        r2eff{i,j} = data{i,j}(:,1);
        err{i,j} = data{i,j}(:,2);
    end
end

%relaxation time
ctimeMQ = 0.04;
ctimeSQ = 0.02;
%number of 180s in each half of relaxation period
n180array{1} = ctimeMQ./(2*taucp{1,1}(:));
n180array{2} = ctimeMQ./(2*taucp{1,2}(:));
n180array{3} = ctimeSQ./(2*taucp{1,3}(:));

for i = 1:flds
    delta{i} = taucp{1,i}(:)./2;
end

for i=1:numres
    if is3state == 1
        x0_res = [dw0(i) 0.5 dwh0(i) 0 r2abc(i,:) 1 15]; %dwcab,dwcac,dwhab,dwhac,r2 cpmg,CEST R1,CEST R2
    else
        x0_res = [dw0(i) dwh0(i) r2abc(i,:) 1 15]; 
    end
   x0 = [x0 x0_res];
end

%ms = MultiStart;
%problem = createOptimProblem('lsqnonlin','objective',@fitfunction_3stateSQMQ_full_linear_global,...
%    'x0',x0,'lb',xlb,'ub',xub);
%[xmin,fmin,flag,outpt,allmins] = run(ms,problem,100);

%x0 = xmin; 
% ############ set up some plotting stuff ############
for i = 1:flds
    itaucp{i} = 1./taucp{i};
end

dat(1).color = 'r'; % 1st field
dat(2).color = 'b';
dat(3).color = 'g';
dat(4).color = 'y';
dat(5).color = 'k';

%set up CEST stuff
freqlist_orig40 = freqs(2:length(freqs));
freqlist_orig25 = freqs25(2:length(freqs25));
freqlist_orig = [freqlist_orig40;freqlist_orig25];
freqlist = freqlist_orig;
datamat = [];
for i = 1:numres
   datamat(:,i) = [relInts(idxlist(i),2:size(relInts,2)) relInts25(idxlist(i),2:size(relInts25,2))]; 
   cslist(:,i) = [0;0;cppm(idxlist(i))];
end
SLdura = 0.4;
SLpow = [powcalib40 calib25pow];
chemcenter = 19.565;
for i = 1:numres
    peakoffset(i) = (cppm(idxlist(i)) - chemcenter)*150.902406439;
end
for i = 1:numres
    datainput{i} = datamat(:,i);
    %datainput2{i} = datamat(:,i);
end
chemshift = cslist(3,:);
freqlist = freqlist_orig; %reset to original freqlist with all points for each res
sllist(1:length(freqlist_orig40)) = SLpow(1);
sllist(length(freqlist_orig40)+1:length(freqlist_orig40)+length(freqlist_orig25)) = SLpow(2);
for i = 1:numres
    sllist_allres{i} = sllist';
    freqlist_allres{i} = freqlist;
end
for i = 1:numres
    numdel = 0;
    for k = 1:size(datainput{i},1)
       %adjust based on 2H decoupling power (removal of sidebands)
       if abs(freqlist_allres{i}(k-numdel)-peakoffset(i)) > 400 && abs(freqlist_allres{i}(k-numdel)-peakoffset(i)) < 600 %remove 2H decoupling sidebands at ~500 Hz
            datainput{i}(k-numdel) = [];
            freqlist_allres{i}(k-numdel) = [];
            sllist_allres{i}(k-numdel) = [];
            numdel = numdel + 1;
       end
    end
end
for i = 1:numres
    % adjust based on number of points in 40 Hz dataset (maximum offset in 40 Hz dataset)
    maxval(i) = max(freqlist_allres{i}(1:71));
end
fityn = 1;
freq = freqlist_allres;
for i = 1:numres
    numpts(i) = size(freq{i},1);
end

% #################### fit data ####################
tic
fprintf('\n\n\nGlobally optimizing parameters to CPMG & CEST data:\n');

if is3state == 1
    [x,resnorm,residual,exitflag,output,lambda,jacobian] = ...
        lsqnonlin(@fitfunction_cpmgCEST_3stateSQMQ_full_linear_global,x0,xlb,xub,options);
    exitflag_orig = exitflag;
else
    [x,resnorm,residual,exitflag,output,lambda,jacobian] = ...
        lsqnonlin(@fitfunction_cpmgCEST_2stateSQMQ_full_linear_global,x0,xlb,xub,options);
    exitflag_orig = exitflag;
end

timer1 = toc;        
fprintf('\nOptimized for %6.2f seconds\n',timer1);

xbest = x;
x0 = xbest;
ci = nlparci(x,residual,'Jacobian',jacobian);

output_orig = output;
chisquared = resnorm;

% Monte Carlo Simulations (2/3-state)

fityn = 1;
%mc = 200;
data_orig = data;
CESTdata_orig = datainput;
rareval = 0;
if is3state == 0
    monte.pa = zeros(1,mc);
    monte.kex = zeros(1,mc);
    monte.dwc = zeros(1,mc);
    monte.dwh = zeros(1,mc);
    monte.R2MQ850 = zeros(1,mc);
    monte.R2MQ600 = zeros(1,mc);
    monte.R2SQ850 = zeros(1,mc);
    monte.R1CEST = zeros(1,mc);
    monte.R2CEST = zeros(1,mc);
    if numres == 2
       monte.dwc2 = zeros(1,mc); 
       monte.dwh2 = zeros(1,mc);
    end
else
    monte.pa = zeros(1,mc);
    monte.pb = zeros(1,mc);
    monte.pc = zeros(1,mc);
    monte.kexab = zeros(1,mc);
    monte.kexac = zeros(1,mc);
    monte.dwcab = zeros(1,mc);
    monte.dwcac = zeros(1,mc);
    monte.R2MQ850 = zeros(1,mc);
    monte.R2MQ600 = zeros(1,mc);
    monte.R2SQ850 = zeros(1,mc);
    monte.R1CEST = zeros(1,mc);
    monte.R2CEST = zeros(1,mc);
end
for i=1:mc
   for k = 1:numres
       for j = 1:flds
           data{k,j}(:,1) = normrnd(data_orig{k,j}(:,1),data_orig{k,j}(:,2));
           %if point with value >3 std from mean is picked, repick points
           while length(data{k,j}(abs(data{k,j}(:,1) - data_orig{k,j}(:,1)) > 3*mean(data_orig{k,j}(:,2)),1)) > 0   
                    data{k,j}(:,1) = normrnd(data_orig{k,j}(:,1),data_orig{k,j}(:,2));
                    rareval = rareval + 1;
           end
       end
       datainput{k} = normrnd(CESTdata_orig{k},1.5*relErr(idxlist(k),1)*ones(size(CESTdata_orig{k},1),size(CESTdata_orig{k},2)));
       %if point with value >3 std from mean is picked, repick points
           while length(datainput{k}(abs(datainput{k} - CESTdata_orig{k}) > 3*1.5*relErr(idxlist(k),1))) > 0   
                    datainput{k} = normrnd(CESTdata_orig{k},1.5*relErr(idxlist(k),1)*ones(size(CESTdata_orig{k},1),size(CESTdata_orig{k},2)));
                    rareval = rareval + 1;
           end
   end
   if is3state == 0
       [x,resnorm,residual,exitflag,output,lambda,jacobian] = ...
        lsqnonlin(@fitfunction_cpmgCEST_2stateSQMQ_full_linear_global,x0,xlb,xub,options);
        monte.pa(i) = x(1);
        monte.kex(i) = x(2);
        monte.dwc(i) = x(3);
        monte.dwh(i) = x(4);
        monte.R2MQ850(i) = x(5);
        monte.R2MQ600(i) = x(6);
        monte.R2SQ850(i) = x(7);
        monte.R1CEST(i) = x(8);
        monte.R2CEST(i) = x(9);
        if numres == 2
            monte.dwc2(i) = x(10);
            monte.dwh2(i) = x(11);
        end
        i
   else
       [x,resnorm,residual,exitflag,output,lambda,jacobian] = ...
        lsqnonlin(@fitfunction_cpmgCEST_3stateSQMQ_full_linear_global,x0,xlb,xub,options);
        monte.pa(i) = x(1);
        monte.pb(i) = x(2)-x(1);
        monte.pc(i) = 1-x(2);
        monte.kexab(i) = x(3);
        monte.kexac(i) = x(4);
        monte.dwcab(i) = x(5);
        monte.dwcac(i) = x(6);
        monte.R2MQ850(i) = x(9);
        monte.R2MQ600(i) = x(10);
        monte.R2SQ850(i) = x(11);
        monte.R1CEST(i) = x(12);
        monte.R2CEST(i) = x(13);
        i
   end
end

%get mean, median, std for params from mc sims
if is3state == 0
    pa_mc = [mean(monte.pa) median(monte.pa) std(monte.pa)];
    kex_mc = [mean(monte.kex) median(monte.kex) std(monte.kex)];
    dwc_mc = [mean(monte.dwc) median(monte.dwc) std(monte.dwc)];
    dwh_mc = [mean(monte.dwh) median(monte.dwh) std(monte.dwh)];
    R2MQ850_mc = [mean(monte.R2MQ850) median(monte.R2MQ850) std(monte.R2MQ850)];
    R2MQ600_mc = [mean(monte.R2MQ600) median(monte.R2MQ600) std(monte.R2MQ600)];
    R2SQ850_mc = [mean(monte.R2SQ850) median(monte.R2SQ850) std(monte.R2SQ850)];
    R1CEST_mc = [mean(monte.R1CEST) median(monte.R1CEST) std(monte.R1CEST)];
    R2CEST_mc = [mean(monte.R2CEST) median(monte.R2CEST) std(monte.R2CEST)];
    if numres == 2
         dwc2_mc = [mean(monte.dwc2) median(monte.dwc2) std(monte.dwc2)];
         dwh2_mc = [mean(monte.dwh2) median(monte.dwh2) std(monte.dwh2)];
    end
else
    pa_mc = [mean(monte.pa) median(monte.pa) std(monte.pa)];
    pb_mc = [mean(monte.pb) median(monte.pb) std(monte.pb)];
    pc_mc = [mean(monte.pc) median(monte.pc) std(monte.pc)];
    kexab_mc = [mean(monte.kexab) median(monte.kexab) std(monte.kexab)];
    kexac_mc = [mean(monte.kexac) median(monte.kexac) std(monte.kexac)];
    dwcab_mc = [mean(monte.dwcab) median(monte.dwcab) std(monte.dwcab)];
    dwcac_mc = [mean(monte.dwcac) median(monte.dwcac) std(monte.dwcac)];
    R2MQ850_mc = [mean(monte.R2MQ850) median(monte.R2MQ850) std(monte.R2MQ850)];
    R2MQ600_mc = [mean(monte.R2MQ600) median(monte.R2MQ600) std(monte.R2MQ600)];
    R2SQ850_mc = [mean(monte.R2SQ850) median(monte.R2SQ850) std(monte.R2SQ850)];
    R1CEST_mc = [mean(monte.R1CEST) median(monte.R1CEST) std(monte.R1CEST)];
    R2CEST_mc = [mean(monte.R2CEST) median(monte.R2CEST) std(monte.R2CEST)];
end

data = data_orig;
datainput = CESTdata_orig;

% ####### plot fitted line #######
clear r2calc;
clear A;
clear Aconj;
clear B1;
clear B2;
clear B3;
clear pa;
clear pc;
clear M0;

fityn = 0;
%finer sampling of CEST profile
freq40 = min(freqs(2:length(freqs))):10:max(freqs(2:length(freqs)));
freq25 = min(freqs25(2:length(freqs25))):10:max(freqs25(2:length(freqs25)));
freq = [];
sllist = [];
freqlist2 = [freq40 freq25];
sllist(1:length(freq40))=SLpow(1);
sllist(length(freq40)+1:length(freq40)+length(freq25))=SLpow(2);
%sllist((size(freq,2)/2)+1,length(freq))=outputparam(9,1);
for i = 1:numres
    sllist_allres{i} = sllist';
    freq{i} = freqlist2';
    numpts(i) = size(freq{i},1);
end
if is3state == 1
    r2calc = fitfunction_cpmgCEST_3stateSQMQ_full_linear_global(xbest); %calc CPMG data from best fit
else
    r2calc = fitfunction_cpmgCEST_2stateSQMQ_full_linear_global(xbest); %calc CPMG data from best fit
end
dataout1 = dataout; %calc CEST data from best fit

plcolor = ['k' 'k' 'k' 'k'];
for j = 1:numres
    %CPMG plots
    for i = 1:flds
        itau{i} = 1./taucp{i};
        fit = [itau{i} r2calc{j,i}'];
        fit = sortrows(fit,1);
        figure
        if isMQ(i) == 1
            errorbar(itaucp{i},r2eff{j,i},err{j,i},err{j,i},'ok','MarkerFaceColor',dat(i).color,'MarkerSize',6);
        else
            errorbar(itaucp{i},r2eff{j,i},err{j,i},err{j,i},'ok','MarkerFaceColor',dat(i).color,'MarkerSize',6);
        end
        hold on
        plot(fit(:,1),fit(:,2),plcolor(i));
        if is3state == 1
            title(strcat(residue{j},' 3 state linear global'));
        else
            title(strcat(residue{j},' 2 state'));
        end
        xlabel('1/\tau_c_p (Hz)');
        ylabel('R_{2,eff} (rad/sec)');
        ylim([min(r2eff{j,i})-2 min(r2eff{j,i})+8])
    end
    %CEST plots
    [a,idx] = ismember(maxval(j),freqlist_allres{j}); %last index of 40 Hz
    figure
    plot(freqlist_allres{j}(1:idx),datainput{j}(1:idx), 'bo', 'MarkerFaceColor', 'b', 'MarkerSize', 2)
    hold on
    plot(freq40,dataout1{j}(1:length(freq40)), 'Color', 'b', 'LineWidth', 1)
    hold off
    axis manual
    hold off
    axis([-1500 1500 -0.1 max(datainput{j})+0.1]);
    set(gca,'fontsize',12)
    %title(strcat('^1^3C Methyl',{' '}, residinfo{i}),{' '},{'40 Hz'});
    title(strcat(residue{j},' 40 Hz'))
    xlabel('^1^3C Frequency Offset (Hz)', 'FontSize', 16, 'FontWeight', 'bold');
    ylabel('Normalized Intensity','FontSize', 16, 'FontWeight', 'bold');
    figure
    plot(freqlist_allres{j}(idx+1:length(freqlist_allres{j})),datainput{j}(idx+1:length(freqlist_allres{j})), 'bo', 'MarkerFaceColor', 'b', 'MarkerSize', 2)
    hold on
    plot(freq25,dataout1{j}(length(freq40)+1:length(freq40)+length(freq25)), 'Color', 'b', 'LineWidth', 1)
    hold off
    axis manual
    hold off
    axis([-1500 1500 -0.1 max(datainput{j})+0.1]);
    set(gca,'fontsize',12)
    %title(strcat('^1^3C Methyl',{' '}, residinfo{i}),{' '},{'25 Hz'});
    title(strcat(residue{j},' 25 Hz'))
    xlabel('^1^3C Frequency Offset (Hz)', 'FontSize', 16, 'FontWeight', 'bold');
    ylabel('Normalized Intensity','FontSize', 16, 'FontWeight', 'bold');
end